package com.quiz.converter.handlers;

import com.quiz.converter.models.Question;
import com.quiz.converter.models.QuestionState;
import com.quiz.converter.models.enums.QuestionType;
import com.quiz.converter.models.enums.QuestionWarningType;

import java.util.List;
import java.util.Optional;

public class QuestionHandler {
    private final QuestionState state;

    public QuestionHandler(QuestionState state) {
        this.state = state;
    }

    public Optional<Question> handleQuestion(String text, List<String> dropdownValues) {
        Optional<Question> question = Optional.empty();
        if (!state.getName().isEmpty()) {
            var validationHandler = new QuestionValidationHandler(state);
            validationHandler.validateQuestion();
            question = Optional.of(state.createQuestion());
            state.cleanState();
        }
        addQuestionType(text, dropdownValues);
        addQuestionName(text);
        addQuestionOptions(text, dropdownValues);
        return question;
    }

    private void addQuestionOptions(String text, List<String> dropdownValues) {
        var lowerCaseText = text.toLowerCase();
        if (matchesRegex(dropdownValues, lowerCaseText, ".*(no)\s*(shuffle).*")) {
            state.setShuffle(false);
        } else if (matchesRegex(dropdownValues, lowerCaseText, ".*(shuffle).*")) {
            state.setShuffle(true);
        }
        if (matchesRegex(dropdownValues, lowerCaseText, ".*(no)\s*(partial)\s*(credit).*")) {
            state.setPartialCredit(false);
        } else if (matchesRegex(dropdownValues, lowerCaseText, ".*(partial)\s*(credit).*")) {
            state.setPartialCredit(true);
        }
    }

    private void addQuestionName(String text) {
        try {
            int endOfNameIndex = !text.contains("-") ? text.indexOf("–") : text.indexOf("-");
            state.setName(text.substring(0, endOfNameIndex));
        } catch (Exception e) {
            state.setName(text);
            state.getWarnings().add(QuestionWarningType.CHECK_QUESTION_NAME);
        }
    }

    private void addQuestionType(String text, List<String> dropdownValues) {
        var lowerCaseText = text.toLowerCase();
        if (matchesRegex(dropdownValues, lowerCaseText, ".*(single)\s*(choice).*")) {
            state.setType(QuestionType.SINGLE_CHOICE);
        } else if (matchesRegex(dropdownValues, lowerCaseText, ".*(multiple)\s*(choice).*|.*(checkbox).*")) {
            state.setType(QuestionType.MULTIPLE_CHOICE);
        } else if (matchesRegex(dropdownValues, lowerCaseText, ".*(text)\s*(match).*|.*(short)\s*(answer).*")) {
            state.setType(QuestionType.TEXT_MATCH);
        } else if (matchesRegex(dropdownValues, lowerCaseText, ".*(regular)\s*(expression).*")) {
            state.setType(QuestionType.REGULAR_EXPRESSION);
        } else {
            state.setType(QuestionType.UNKNOWN);
        }
    }

    private boolean matchesRegex(List<String> dropdownValues, String lowerCaseText, String regex) {
        return lowerCaseText.matches(regex) || dropdownValues.stream().anyMatch(s -> s.matches(regex));
    }
}
